"""
    Lookup/Ibis web service API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.2
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from identitylib.lookup_client.api_client import ApiClient, Endpoint as _Endpoint
from identitylib.lookup_client.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from identitylib.lookup_client.model.group_all_groups200_response import GroupAllGroups200Response
from identitylib.lookup_client.model.group_all_groups_default_response import GroupAllGroupsDefaultResponse
from identitylib.lookup_client.model.group_get_cancelled_members200_response import GroupGetCancelledMembers200Response
from identitylib.lookup_client.model.group_search_count200_response import GroupSearchCount200Response
from identitylib.lookup_client.model.institution_all_attribute_schemes200_response import InstitutionAllAttributeSchemes200Response
from identitylib.lookup_client.model.institution_all_insts200_response import InstitutionAllInsts200Response
from identitylib.lookup_client.model.institution_get_attribute200_response import InstitutionGetAttribute200Response
from identitylib.lookup_client.model.institution_get_attributes200_response import InstitutionGetAttributes200Response
from identitylib.lookup_client.model.person_get_person200_response import PersonGetPerson200Response
from identitylib.lookup_client.model.person_is_member_of_group200_response import PersonIsMemberOfGroup200Response


class PersonApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client
        self.person_all_attribute_schemes_endpoint = _Endpoint(
            settings={
                'response_type': (InstitutionAllAttributeSchemes200Response,),
                'auth': [
                    'APIGatewayAuth',
                    'APIGatewayAuth'
                ],
                'endpoint_path': '/person/all-attr-schemes',
                'operation_id': 'person_all_attribute_schemes',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                ],
                'required': [],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                },
                'attribute_map': {
                },
                'location_map': {
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.person_all_people_endpoint = _Endpoint(
            settings={
                'response_type': (GroupGetCancelledMembers200Response,),
                'auth': [
                    'APIGatewayAuth',
                    'APIGatewayAuth'
                ],
                'endpoint_path': '/person/all-people',
                'operation_id': 'person_all_people',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'include_cancelled',
                    'identifier',
                    'limit',
                    'fetch',
                ],
                'required': [],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'include_cancelled':
                        (bool,),
                    'identifier':
                        (str,),
                    'limit':
                        (int,),
                    'fetch':
                        (str,),
                },
                'attribute_map': {
                    'include_cancelled': 'includeCancelled',
                    'identifier': 'identifier',
                    'limit': 'limit',
                    'fetch': 'fetch',
                },
                'location_map': {
                    'include_cancelled': 'query',
                    'identifier': 'query',
                    'limit': 'query',
                    'fetch': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.person_get_attribute_endpoint = _Endpoint(
            settings={
                'response_type': (InstitutionGetAttribute200Response,),
                'auth': [
                    'APIGatewayAuth',
                    'APIGatewayAuth'
                ],
                'endpoint_path': '/person/{scheme}/{identifier}/{attrid}',
                'operation_id': 'person_get_attribute',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'scheme',
                    'identifier',
                    'attrid',
                ],
                'required': [
                    'scheme',
                    'identifier',
                    'attrid',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'scheme':
                        (str,),
                    'identifier':
                        (str,),
                    'attrid':
                        (int,),
                },
                'attribute_map': {
                    'scheme': 'scheme',
                    'identifier': 'identifier',
                    'attrid': 'attrid',
                },
                'location_map': {
                    'scheme': 'path',
                    'identifier': 'path',
                    'attrid': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.person_get_attributes_endpoint = _Endpoint(
            settings={
                'response_type': (InstitutionGetAttributes200Response,),
                'auth': [
                    'APIGatewayAuth',
                    'APIGatewayAuth'
                ],
                'endpoint_path': '/person/{scheme}/{identifier}/get-attributes',
                'operation_id': 'person_get_attributes',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'scheme',
                    'identifier',
                    'attrs',
                ],
                'required': [
                    'scheme',
                    'identifier',
                    'attrs',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'scheme':
                        (str,),
                    'identifier':
                        (str,),
                    'attrs':
                        (str,),
                },
                'attribute_map': {
                    'scheme': 'scheme',
                    'identifier': 'identifier',
                    'attrs': 'attrs',
                },
                'location_map': {
                    'scheme': 'path',
                    'identifier': 'path',
                    'attrs': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.person_get_groups_endpoint = _Endpoint(
            settings={
                'response_type': (GroupAllGroups200Response,),
                'auth': [
                    'APIGatewayAuth',
                    'APIGatewayAuth'
                ],
                'endpoint_path': '/person/{scheme}/{identifier}/groups',
                'operation_id': 'person_get_groups',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'scheme',
                    'identifier',
                    'fetch',
                ],
                'required': [
                    'scheme',
                    'identifier',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'scheme':
                        (str,),
                    'identifier':
                        (str,),
                    'fetch':
                        (str,),
                },
                'attribute_map': {
                    'scheme': 'scheme',
                    'identifier': 'identifier',
                    'fetch': 'fetch',
                },
                'location_map': {
                    'scheme': 'path',
                    'identifier': 'path',
                    'fetch': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.person_get_insts_endpoint = _Endpoint(
            settings={
                'response_type': (InstitutionAllInsts200Response,),
                'auth': [
                    'APIGatewayAuth',
                    'APIGatewayAuth'
                ],
                'endpoint_path': '/person/{scheme}/{identifier}/insts',
                'operation_id': 'person_get_insts',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'scheme',
                    'identifier',
                    'fetch',
                ],
                'required': [
                    'scheme',
                    'identifier',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'scheme':
                        (str,),
                    'identifier':
                        (str,),
                    'fetch':
                        (str,),
                },
                'attribute_map': {
                    'scheme': 'scheme',
                    'identifier': 'identifier',
                    'fetch': 'fetch',
                },
                'location_map': {
                    'scheme': 'path',
                    'identifier': 'path',
                    'fetch': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.person_get_managed_groups_endpoint = _Endpoint(
            settings={
                'response_type': (GroupAllGroups200Response,),
                'auth': [
                    'APIGatewayAuth',
                    'APIGatewayAuth'
                ],
                'endpoint_path': '/person/{scheme}/{identifier}/manages-groups',
                'operation_id': 'person_get_managed_groups',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'scheme',
                    'identifier',
                    'fetch',
                ],
                'required': [
                    'scheme',
                    'identifier',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'scheme':
                        (str,),
                    'identifier':
                        (str,),
                    'fetch':
                        (str,),
                },
                'attribute_map': {
                    'scheme': 'scheme',
                    'identifier': 'identifier',
                    'fetch': 'fetch',
                },
                'location_map': {
                    'scheme': 'path',
                    'identifier': 'path',
                    'fetch': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.person_get_managed_insts_endpoint = _Endpoint(
            settings={
                'response_type': (InstitutionAllInsts200Response,),
                'auth': [
                    'APIGatewayAuth',
                    'APIGatewayAuth'
                ],
                'endpoint_path': '/person/{scheme}/{identifier}/manages-insts',
                'operation_id': 'person_get_managed_insts',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'scheme',
                    'identifier',
                    'fetch',
                ],
                'required': [
                    'scheme',
                    'identifier',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'scheme':
                        (str,),
                    'identifier':
                        (str,),
                    'fetch':
                        (str,),
                },
                'attribute_map': {
                    'scheme': 'scheme',
                    'identifier': 'identifier',
                    'fetch': 'fetch',
                },
                'location_map': {
                    'scheme': 'path',
                    'identifier': 'path',
                    'fetch': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.person_get_person_endpoint = _Endpoint(
            settings={
                'response_type': (PersonGetPerson200Response,),
                'auth': [
                    'APIGatewayAuth',
                    'APIGatewayAuth'
                ],
                'endpoint_path': '/person/{scheme}/{identifier}',
                'operation_id': 'person_get_person',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'scheme',
                    'identifier',
                    'fetch',
                ],
                'required': [
                    'scheme',
                    'identifier',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'scheme':
                        (str,),
                    'identifier':
                        (str,),
                    'fetch':
                        (str,),
                },
                'attribute_map': {
                    'scheme': 'scheme',
                    'identifier': 'identifier',
                    'fetch': 'fetch',
                },
                'location_map': {
                    'scheme': 'path',
                    'identifier': 'path',
                    'fetch': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.person_is_member_of_group_endpoint = _Endpoint(
            settings={
                'response_type': (PersonIsMemberOfGroup200Response,),
                'auth': [
                    'APIGatewayAuth',
                    'APIGatewayAuth'
                ],
                'endpoint_path': '/person/{scheme}/{identifier}/is-member-of-group/{groupid}',
                'operation_id': 'person_is_member_of_group',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'scheme',
                    'identifier',
                    'groupid',
                ],
                'required': [
                    'scheme',
                    'identifier',
                    'groupid',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'scheme':
                        (str,),
                    'identifier':
                        (str,),
                    'groupid':
                        (str,),
                },
                'attribute_map': {
                    'scheme': 'scheme',
                    'identifier': 'identifier',
                    'groupid': 'groupid',
                },
                'location_map': {
                    'scheme': 'path',
                    'identifier': 'path',
                    'groupid': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.person_is_member_of_inst_endpoint = _Endpoint(
            settings={
                'response_type': (PersonIsMemberOfGroup200Response,),
                'auth': [
                    'APIGatewayAuth',
                    'APIGatewayAuth'
                ],
                'endpoint_path': '/person/{scheme}/{identifier}/is-member-of-inst/{instid}',
                'operation_id': 'person_is_member_of_inst',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'scheme',
                    'identifier',
                    'instid',
                ],
                'required': [
                    'scheme',
                    'identifier',
                    'instid',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'scheme':
                        (str,),
                    'identifier':
                        (str,),
                    'instid':
                        (str,),
                },
                'attribute_map': {
                    'scheme': 'scheme',
                    'identifier': 'identifier',
                    'instid': 'instid',
                },
                'location_map': {
                    'scheme': 'path',
                    'identifier': 'path',
                    'instid': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.person_list_people_endpoint = _Endpoint(
            settings={
                'response_type': (GroupGetCancelledMembers200Response,),
                'auth': [
                    'APIGatewayAuth',
                    'APIGatewayAuth'
                ],
                'endpoint_path': '/person/list',
                'operation_id': 'person_list_people',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'crsids',
                    'fetch',
                ],
                'required': [
                    'crsids',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'crsids':
                        (str,),
                    'fetch':
                        (str,),
                },
                'attribute_map': {
                    'crsids': 'crsids',
                    'fetch': 'fetch',
                },
                'location_map': {
                    'crsids': 'query',
                    'fetch': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.person_modified_people_endpoint = _Endpoint(
            settings={
                'response_type': (GroupGetCancelledMembers200Response,),
                'auth': [
                    'APIGatewayAuth',
                    'APIGatewayAuth'
                ],
                'endpoint_path': '/person/modified-people',
                'operation_id': 'person_modified_people',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'min_tx_id',
                    'max_tx_id',
                    'crsids',
                    'include_cancelled',
                    'membership_changes',
                    'inst_name_changes',
                    'fetch',
                ],
                'required': [
                    'min_tx_id',
                    'max_tx_id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'min_tx_id':
                        (int,),
                    'max_tx_id':
                        (int,),
                    'crsids':
                        (str,),
                    'include_cancelled':
                        (bool,),
                    'membership_changes':
                        (bool,),
                    'inst_name_changes':
                        (bool,),
                    'fetch':
                        (str,),
                },
                'attribute_map': {
                    'min_tx_id': 'minTxId',
                    'max_tx_id': 'maxTxId',
                    'crsids': 'crsids',
                    'include_cancelled': 'includeCancelled',
                    'membership_changes': 'membershipChanges',
                    'inst_name_changes': 'instNameChanges',
                    'fetch': 'fetch',
                },
                'location_map': {
                    'min_tx_id': 'query',
                    'max_tx_id': 'query',
                    'crsids': 'query',
                    'include_cancelled': 'query',
                    'membership_changes': 'query',
                    'inst_name_changes': 'query',
                    'fetch': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.person_search_endpoint = _Endpoint(
            settings={
                'response_type': (GroupGetCancelledMembers200Response,),
                'auth': [
                    'APIGatewayAuth',
                    'APIGatewayAuth'
                ],
                'endpoint_path': '/person/search',
                'operation_id': 'person_search',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'query',
                    'approx_matches',
                    'include_cancelled',
                    'mis_status',
                    'attributes',
                    'offset',
                    'limit',
                    'order_by',
                    'fetch',
                ],
                'required': [
                    'query',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'query':
                        (str,),
                    'approx_matches':
                        (bool,),
                    'include_cancelled':
                        (bool,),
                    'mis_status':
                        (str,),
                    'attributes':
                        (str,),
                    'offset':
                        (int,),
                    'limit':
                        (int,),
                    'order_by':
                        (str,),
                    'fetch':
                        (str,),
                },
                'attribute_map': {
                    'query': 'query',
                    'approx_matches': 'approxMatches',
                    'include_cancelled': 'includeCancelled',
                    'mis_status': 'misStatus',
                    'attributes': 'attributes',
                    'offset': 'offset',
                    'limit': 'limit',
                    'order_by': 'orderBy',
                    'fetch': 'fetch',
                },
                'location_map': {
                    'query': 'query',
                    'approx_matches': 'query',
                    'include_cancelled': 'query',
                    'mis_status': 'query',
                    'attributes': 'query',
                    'offset': 'query',
                    'limit': 'query',
                    'order_by': 'query',
                    'fetch': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.person_search_count_endpoint = _Endpoint(
            settings={
                'response_type': (GroupSearchCount200Response,),
                'auth': [
                    'APIGatewayAuth',
                    'APIGatewayAuth'
                ],
                'endpoint_path': '/person/search-count',
                'operation_id': 'person_search_count',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'query',
                    'approx_matches',
                    'include_cancelled',
                    'mis_status',
                    'attributes',
                ],
                'required': [
                    'query',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'query':
                        (str,),
                    'approx_matches':
                        (bool,),
                    'include_cancelled':
                        (bool,),
                    'mis_status':
                        (str,),
                    'attributes':
                        (str,),
                },
                'attribute_map': {
                    'query': 'query',
                    'approx_matches': 'approxMatches',
                    'include_cancelled': 'includeCancelled',
                    'mis_status': 'misStatus',
                    'attributes': 'attributes',
                },
                'location_map': {
                    'query': 'query',
                    'approx_matches': 'query',
                    'include_cancelled': 'query',
                    'mis_status': 'query',
                    'attributes': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )

    def person_all_attribute_schemes(
        self,
        **kwargs
    ):
        """Return a list of all the person attribute schemes available.  # noqa: E501

        Return a list of all the person attribute schemes available. The `schemeid` values of these schemes may be used in the `fetch` parameter of other methods that return people.  NOTE: Some of these attribute schemes are not currently used (no people have attribute values in the scheme). These schemes are reserved for possible future use.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.person_all_attribute_schemes(async_req=True)
        >>> result = thread.get()


        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            InstitutionAllAttributeSchemes200Response
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        return self.person_all_attribute_schemes_endpoint.call_with_http_info(**kwargs)

    def person_all_people(
        self,
        **kwargs
    ):
        """Return a list of all people (in batches).  # noqa: E501

        Return a list of all people (in batches).  The results are sorted by identifier, starting with the first person after the person with the specified identifier. Thus, to iterate over all people, pass a `null` identifier to get the first batch of people, then pass the last identifier from the previous batch to get the next batch, and repeat until no more people are returned.  By default, only a few basic details about each person are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.person_all_people(async_req=True)
        >>> result = thread.get()


        Keyword Args:
            include_cancelled (bool): Flag to allow cancelled people to be included (people who are no longer members of the University). Defaults to `false`.. [optional]
            identifier (str): The identifier (CRSid) of the person to start after, or `null` to start from the first person.. [optional]
            limit (int): The maximum number of people to return. Defaults to 100.. [optional]
            fetch (str): A comma-separated list of any additional attributes or references to fetch.. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            GroupGetCancelledMembers200Response
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        return self.person_all_people_endpoint.call_with_http_info(**kwargs)

    def person_get_attribute(
        self,
        scheme,
        identifier,
        attrid,
        **kwargs
    ):
        """Get a specific attribute of a person.  # noqa: E501

        Get a specific attribute of a person.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.person_get_attribute(scheme, identifier, attrid, async_req=True)
        >>> result = thread.get()

        Args:
            scheme (str): The person identifier scheme. Typically this should be `\"crsid\"`, but other identifier schemes may be available in the future, such as `\"usn\"` or `\"staffNumber\"`.
            identifier (str): The identifier of the person (typically their CRSid).
            attrid (int): The ID of the attribute to fetch.

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            InstitutionGetAttribute200Response
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['scheme'] = \
            scheme
        kwargs['identifier'] = \
            identifier
        kwargs['attrid'] = \
            attrid
        return self.person_get_attribute_endpoint.call_with_http_info(**kwargs)

    def person_get_attributes(
        self,
        scheme,
        identifier,
        attrs,
        **kwargs
    ):
        """Get one or more (possibly multi-valued) attributes of a person.  # noqa: E501

        Get one or more (possibly multi-valued) attributes of a person. The returned attributes are sorted by attribute scheme precedence and then attribute precedence.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.person_get_attributes(scheme, identifier, attrs, async_req=True)
        >>> result = thread.get()

        Args:
            scheme (str): The person identifier scheme. Typically this should be `\"crsid\"`, but other identifier schemes may be available in the future, such as `\"usn\"` or `\"staffNumber\"`.
            identifier (str): The identifier of the person (typically their CRSid).
            attrs (str): The attribute scheme(s) to fetch. This may include any number of the attributes or pseudo-attributes, but it may not include references or attribute chains (see the documentation for the `fetch` parameter in this class).

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            InstitutionGetAttributes200Response
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['scheme'] = \
            scheme
        kwargs['identifier'] = \
            identifier
        kwargs['attrs'] = \
            attrs
        return self.person_get_attributes_endpoint.call_with_http_info(**kwargs)

    def person_get_groups(
        self,
        scheme,
        identifier,
        **kwargs
    ):
        """Get all the groups to which the specified person belongs, including indirect group memberships, via groups that include other groups.  # noqa: E501

        Get all the groups to which the specified person belongs, including indirect group memberships, via groups that include other groups. The returned list of groups is sorted by groupid.  Note that some group memberships may not be visible to you. This method will only return those group memberships that you have permission to see.  By default, only a few basic details about each group are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references of each group.  NOTE: This method will not include cancelled groups.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.person_get_groups(scheme, identifier, async_req=True)
        >>> result = thread.get()

        Args:
            scheme (str): The person identifier scheme. Typically this should be `\"crsid\"`, but other identifier schemes may be available in the future, such as `\"usn\"` or `\"staffNumber\"`.
            identifier (str): The identifier of the person (typically their CRSid).

        Keyword Args:
            fetch (str): A comma-separated list of any additional attributes or references to fetch.. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            GroupAllGroups200Response
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['scheme'] = \
            scheme
        kwargs['identifier'] = \
            identifier
        return self.person_get_groups_endpoint.call_with_http_info(**kwargs)

    def person_get_insts(
        self,
        scheme,
        identifier,
        **kwargs
    ):
        """Get all the institutions to which the specified person belongs.  # noqa: E501

        Get all the institutions to which the specified person belongs. The returned list of institutions is sorted by name.  By default, only a few basic details about each institution are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references of each institution.  NOTE: This method will not include cancelled institutions.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.person_get_insts(scheme, identifier, async_req=True)
        >>> result = thread.get()

        Args:
            scheme (str): The person identifier scheme. Typically this should be `\"crsid\"`, but other identifier schemes may be available in the future, such as `\"usn\"` or `\"staffNumber\"`.
            identifier (str): The identifier of the person (typically their CRSid).

        Keyword Args:
            fetch (str): A comma-separated list of any additional attributes or references to fetch.. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            InstitutionAllInsts200Response
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['scheme'] = \
            scheme
        kwargs['identifier'] = \
            identifier
        return self.person_get_insts_endpoint.call_with_http_info(**kwargs)

    def person_get_managed_groups(
        self,
        scheme,
        identifier,
        **kwargs
    ):
        """Get all the groups that the specified person has persmission to edit.  # noqa: E501

        Get all the groups that the specified person has persmission to edit. The returned list of groups is sorted by groupid.  Note that some group memberships may not be visible to you. This method will only include groups for which you have persmission to see the applicable manager group memberships.  By default, only a few basic details about each group are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references of each group.  NOTE: This method will not include cancelled groups.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.person_get_managed_groups(scheme, identifier, async_req=True)
        >>> result = thread.get()

        Args:
            scheme (str): The person identifier scheme. Typically this should be `\"crsid\"`, but other identifier schemes may be available in the future, such as `\"usn\"` or `\"staffNumber\"`.
            identifier (str): The identifier of the person (typically their CRSid).

        Keyword Args:
            fetch (str): A comma-separated list of any additional attributes or references to fetch.. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            GroupAllGroups200Response
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['scheme'] = \
            scheme
        kwargs['identifier'] = \
            identifier
        return self.person_get_managed_groups_endpoint.call_with_http_info(**kwargs)

    def person_get_managed_insts(
        self,
        scheme,
        identifier,
        **kwargs
    ):
        """Get all the institutions that the specified person has permission to edit.  # noqa: E501

        Get all the institutions that the specified person has permission to edit. The returned list of institutions is sorted by name.  Note that some group memberships may not be visible to you. This method will only include institutions for which you have permission to see the applicable editor group memberships.  By default, only a few basic details about each institution are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references of each institution.  NOTE: This method will not include cancelled institutions.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.person_get_managed_insts(scheme, identifier, async_req=True)
        >>> result = thread.get()

        Args:
            scheme (str): The person identifier scheme. Typically this should be `\"crsid\"`, but other identifier schemes may be available in the future, such as `\"usn\"` or `\"staffNumber\"`.
            identifier (str): The identifier of the person (typically their CRSid).

        Keyword Args:
            fetch (str): A comma-separated list of any additional attributes or references to fetch.. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            InstitutionAllInsts200Response
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['scheme'] = \
            scheme
        kwargs['identifier'] = \
            identifier
        return self.person_get_managed_insts_endpoint.call_with_http_info(**kwargs)

    def person_get_person(
        self,
        scheme,
        identifier,
        **kwargs
    ):
        """Get the person with the specified identifier.  # noqa: E501

        Get the person with the specified identifier.  By default, only a few basic details about the person are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references of the person.  NOTE: The person returned may be a cancelled person. It is the caller's repsonsibility to check its cancelled flag.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.person_get_person(scheme, identifier, async_req=True)
        >>> result = thread.get()

        Args:
            scheme (str): The person identifier scheme. Typically this should be `\"crsid\"`, but other identifier schemes may be available in the future, such as `\"usn\"` or `\"staffNumber\"`.
            identifier (str): The identifier of the person to fetch (typically their CRSid).

        Keyword Args:
            fetch (str): A comma-separated list of any additional attributes or references to fetch.. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            PersonGetPerson200Response
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['scheme'] = \
            scheme
        kwargs['identifier'] = \
            identifier
        return self.person_get_person_endpoint.call_with_http_info(**kwargs)

    def person_is_member_of_group(
        self,
        scheme,
        identifier,
        groupid,
        **kwargs
    ):
        """Test if the specified person is a member of the specified group.  # noqa: E501

        Test if the specified person is a member of the specified group.  NOTE: This may be used with cancelled people and groups.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.person_is_member_of_group(scheme, identifier, groupid, async_req=True)
        >>> result = thread.get()

        Args:
            scheme (str): The person identifier scheme. Typically this should be `\"crsid\"`, but other identifier schemes may be available in the future, such as `\"usn\"` or `\"staffNumber\"`.
            identifier (str): The identifier of the person (typically their CRSid).
            groupid (str): The ID or name of the group.

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            PersonIsMemberOfGroup200Response
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['scheme'] = \
            scheme
        kwargs['identifier'] = \
            identifier
        kwargs['groupid'] = \
            groupid
        return self.person_is_member_of_group_endpoint.call_with_http_info(**kwargs)

    def person_is_member_of_inst(
        self,
        scheme,
        identifier,
        instid,
        **kwargs
    ):
        """Test if the specified person is a member of the specified institution.  # noqa: E501

        Test if the specified person is a member of the specified institution.  NOTE: This may be used with cancelled people and institutions, but it will not include cancelled membership groups.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.person_is_member_of_inst(scheme, identifier, instid, async_req=True)
        >>> result = thread.get()

        Args:
            scheme (str): The person identifier scheme. Typically this should be `\"crsid\"`, but other identifier schemes may be available in the future, such as `\"usn\"` or `\"staffNumber\"`.
            identifier (str): The identifier of the person (typically their CRSid).
            instid (str): The ID of the institution.

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            PersonIsMemberOfGroup200Response
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['scheme'] = \
            scheme
        kwargs['identifier'] = \
            identifier
        kwargs['instid'] = \
            instid
        return self.person_is_member_of_inst_endpoint.call_with_http_info(**kwargs)

    def person_list_people(
        self,
        crsids,
        **kwargs
    ):
        """Get the people with the specified identifiers (typically CRSids).  # noqa: E501

        Get the people with the specified identifiers (typically CRSids).  Each identifier may be either a CRSid, or an identifier from another identifier scheme, prefixed with that scheme's name and a slash. For example `\"mug99\"` or `\"usn/123456789\"`.  By default, only a few basic details about each person are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references.  The results are sorted by identifier scheme and value.  NOTE: The number of people that may be fetched in a single call is limited by the URL path length limit (around 8000 characters). A CRSid is up to 7 characters long, and other identifiers are typically longer, since they must also include the identifier scheme. Thus the number of people that this method may fetch is typically limited to a few hundred.  NOTE: The people returned may include cancelled people. It is the caller's repsonsibility to check their cancelled flags.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.person_list_people(crsids, async_req=True)
        >>> result = thread.get()

        Args:
            crsids (str): A comma-separated list of identifiers.

        Keyword Args:
            fetch (str): A comma-separated list of any additional attributes or references to fetch.. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            GroupGetCancelledMembers200Response
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['crsids'] = \
            crsids
        return self.person_list_people_endpoint.call_with_http_info(**kwargs)

    def person_modified_people(
        self,
        min_tx_id,
        max_tx_id,
        **kwargs
    ):
        """Find all people modified between the specified pair of transactions.  # noqa: E501

        Find all people modified between the specified pair of transactions.  The transaction IDs specified should be the IDs from two different requests for the last (most recent) transaction ID, made at different times, that returned different values, indicating that some Lookup data was modified in the period between the two requests. This method then determines which (if any) people were affected.  By default, only a few basic details about each person are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references.  NOTE: All data returned reflects the latest available data about each person. It is not possible to query for old data, or more detailed information about the specific changes made.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.person_modified_people(min_tx_id, max_tx_id, async_req=True)
        >>> result = thread.get()

        Args:
            min_tx_id (int): Include modifications made in transactions after (but not including) this one.
            max_tx_id (int): Include modifications made in transactions up to and including this one.

        Keyword Args:
            crsids (str): Only include people with identifiers in this list. By default, all modified people will be included.. [optional]
            include_cancelled (bool): Include cancelled people (people who are no longer members of the University). By default, cancelled people are excluded.. [optional]
            membership_changes (bool): Include people whose group or institutional memberships have changed. By default, only people whose attributes have been directly modified are included.. [optional]
            inst_name_changes (bool): Include people who are members of instituions whose names have changed. This will also cause people whose group or institutional memberships have changed to be included. By default, changes to institution names do not propagate to people.. [optional]
            fetch (str): A comma-separated list of any additional attributes or references to fetch.. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            GroupGetCancelledMembers200Response
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['min_tx_id'] = \
            min_tx_id
        kwargs['max_tx_id'] = \
            max_tx_id
        return self.person_modified_people_endpoint.call_with_http_info(**kwargs)

    def person_search(
        self,
        query,
        **kwargs
    ):
        """Search for people using a free text query string.  # noqa: E501

        Search for people using a free text query string. This is the same search function that is used in the Lookup web application.  By default, only a few basic details about each person are returned, but the optional `fetch` parameter may be used to fetch additional attributes or references.  NOTE: If the query string starts with the prefix `\"person:\"`, it is treated as an [LQL query](/lql), allowing more advanced searches. An LQL query will ignore the `approxMatches` and `attributes` parameters, but it will respect the values of `includeCancelled` and `misStatus`. In addition, an LQL query will ignore the `orderBy` parameter, since LQL queries always return results in ID order.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.person_search(query, async_req=True)
        >>> result = thread.get()

        Args:
            query (str): The search string.

        Keyword Args:
            approx_matches (bool): Flag to enable more approximate matching in the search, causing more results to be returned. Defaults to `false`. This is ignored for LQL queries.. [optional]
            include_cancelled (bool): Flag to allow cancelled people to be included (people who are no longer members of the University). Defaults to `false`.. [optional]
            mis_status (str): The type of people to search for. This may be  * `\"staff\"` - only include people whose MIS status is   `\"\"` (empty string), `\"staff\"`, or   `\"staff,student\"`.  * `\"student\"` - only include people whose MIS status is set to   `\"student\"` or `\"staff,student\"`.  Otherwise all matching people will be included (the default). Note that the `\"staff\"` and `\"student\"` options are not mutually exclusive.. [optional]
            attributes (str): A comma-separated list of attributes to consider when searching. If this is `null` (the default) then all attribute schemes marked as searchable will be included. This is ignored for LQL queries.. [optional]
            offset (int): The number of results to skip at the start of the search. Defaults to 0.. [optional]
            limit (int): The maximum number of results to return. Defaults to 100.. [optional]
            order_by (str): The order in which to list the results. This may be either `\"identifier\"` or `\"surname\"` (the default for non-LQL queries). This is ignored for LQL queries, which always return results in identifier order.. [optional]
            fetch (str): A comma-separated list of any additional attributes or references to fetch.. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            GroupGetCancelledMembers200Response
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['query'] = \
            query
        return self.person_search_endpoint.call_with_http_info(**kwargs)

    def person_search_count(
        self,
        query,
        **kwargs
    ):
        """Count the number of people that would be returned by a search using a free text query string.  # noqa: E501

        Count the number of people that would be returned by a search using a free text query string.  NOTE: If the query string starts with the prefix `\"person:\"`, it is treated as an [LQL query](/lql), allowing more advanced searches. An LQL query will ignore the `approxMatches` and `attributes` parameters, but it will respect the values of `includeCancelled` and `misStatus`.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.person_search_count(query, async_req=True)
        >>> result = thread.get()

        Args:
            query (str): The search string.

        Keyword Args:
            approx_matches (bool): Flag to enable more approximate matching in the search, causing more results to be returned. Defaults to `false`. This is ignored for LQL queries.. [optional]
            include_cancelled (bool): Flag to allow cancelled people to be included (people who are no longer members of the University). Defaults to `false`.. [optional]
            mis_status (str): The type of people to search for. This may be  * `\"staff\"` - only include people whose MIS status is   `\"\"` (empty string), `\"staff\"`, or   `\"staff,student\"`.  * `\"student\"` - only include people whose MIS status is set to   `\"student\"` or `\"staff,student\"`.  Otherwise all matching people will be included (the default). Note that the `\"staff\"` and `\"student\"` options are not mutually exclusive.. [optional]
            attributes (str): A comma-separated list of attributes to consider when searching. If this is `null` (the default) then all attribute schemes marked as searchable will be included. This is ignored for LQL queries.. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            GroupSearchCount200Response
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['query'] = \
            query
        return self.person_search_count_endpoint.call_with_http_info(**kwargs)




# flake8: noqa

"""
    University Human Resources API

    This API exposes identity information from the University Human Resources system. The purpose of this API is to give access to basic identity information about staff members for use by downstream systems, therefore the data exposed is limited to basic information about who a staff member is and details of any affiliations they have.  This API uses the following identifiers to link people to person-records held within different systems:  * `person.v1.human-resources.university.identifiers.cam.ac.uk` - the university human resources (uhr) identifier of     this person - also known as the staff number of HRN. * `v1.person.identifiers.cam.ac.uk` - the common registration scheme identifier (CRSid) of     this person.  Additionally this API uses the following identifiers to link people to institutions or entities which a person may have an affiliation to:  * `institution.v1.human-resources.university.identifiers.cam.ac.uk` - the university human resources (uhr) institution     identifier. This encompasses departments within the University HR system     (currently CHRIS). * `institution.v1.student-records.university.identifiers.cam.ac.uk` - the student records institution identifier. This     encompasses colleges within the University Student Records system (currently CamSIS).  Affiliations can be used to indicate an institution that a person holds a position within, with the `status` indicating the job role that is held and the scheme of the affiliation being `institution.v1.human-resources.university.identifiers.cam.ac.uk`. Additionally an affiliation can indicate a staff member's line manager with the scheme being `person.v1.human-resources.university.identifiers.cam.ac.uk` and the status being `report`. A staff member can have multiple institution affiliations and multiple report affiliations.  This API is currently backed by data from CHRIS, exposing a view which is updated hourly.    # noqa: E501

    The version of the OpenAPI document: v1alpha2
    Generated by: https://openapi-generator.tech
"""


__version__ = "1.0.0"

# import ApiClient
from identitylib.hr_client.api_client import ApiClient

# import Configuration
from identitylib.hr_client.configuration import Configuration

# import exceptions
from identitylib.hr_client.exceptions import OpenApiException
from identitylib.hr_client.exceptions import ApiAttributeError
from identitylib.hr_client.exceptions import ApiTypeError
from identitylib.hr_client.exceptions import ApiValueError
from identitylib.hr_client.exceptions import ApiKeyError
from identitylib.hr_client.exceptions import ApiException



# flake8: noqa

"""
    Lookup/Ibis web service API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.2
    Generated by: https://openapi-generator.tech
"""


__version__ = "1.0.0"

# import ApiClient
from identitylib.lookup_client.api_client import ApiClient

# import Configuration
from identitylib.lookup_client.configuration import Configuration

# import exceptions
from identitylib.lookup_client.exceptions import OpenApiException
from identitylib.lookup_client.exceptions import ApiAttributeError
from identitylib.lookup_client.exceptions import ApiTypeError
from identitylib.lookup_client.exceptions import ApiValueError
from identitylib.lookup_client.exceptions import ApiKeyError
from identitylib.lookup_client.exceptions import ApiException



"""
    University Student API

    This API exposes identity information about university students. The purpose of this API is to give access to basic identity information about university students for use by downstream systems, therefore the data exposed is limited to basic information about who a student is and which institutions they are affiliated with.  This API uses the following identifiers to link people to person-records held within different systems:  * `person.v1.student-records.university.identifiers.cam.ac.uk` - the university student (ust) identifier of this person -   also referred to as USN  Additionally this API uses the following identifiers to link people to institutions or entities which a person may have an affiliation to:  * `institution.v1.student-records.university.identifiers.cam.ac.uk` - the university student (ust) institution identifier.     This encompasses colleges, departments and faculties. The     [CamSIS Coding Manual](https://www.camsis.cam.ac.uk/technical/camsis-coding-manual)     lists the available values for this affiliation (under A01 and M01). * `academic-plan.v1.student-records.university.identifiers.cam.ac.uk` - the university student (ust) academic plan     identifier. The     [CamSIS Coding Manual](https://www.camsis.cam.ac.uk/technical/camsis-coding-manual)     lists the available values for this affiliation (under D07).  Both affiliations provide a `status` field which indicate a student's status within an affiliated entity. This `status` field is the short representation of the academic career field held in CamSIS. The [CamSIS Coding Manual](https://www.camsis.cam.ac.uk/technical/camsis-coding-manual) lists the available values for the `status` field (under D05).  This API is currently backed by data from CamSIS, exposing a view of data that is refreshed nightly.    # noqa: E501

    The version of the OpenAPI document: v1alpha2
    Generated by: https://openapi-generator.tech
"""


class OpenApiException(Exception):
    """The base exception class for all OpenAPIExceptions"""


class ApiTypeError(OpenApiException, TypeError):
    def __init__(self, msg, path_to_item=None, valid_classes=None,
                 key_type=None):
        """ Raises an exception for TypeErrors

        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (list): a list of keys an indices to get to the
                                 current_item
                                 None if unset
            valid_classes (tuple): the primitive classes that current item
                                   should be an instance of
                                   None if unset
            key_type (bool): False if our value is a value in a dict
                             True if it is a key in a dict
                             False if our item is an item in a list
                             None if unset
        """
        self.path_to_item = path_to_item
        self.valid_classes = valid_classes
        self.key_type = key_type
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiTypeError, self).__init__(full_msg)


class ApiValueError(OpenApiException, ValueError):
    def __init__(self, msg, path_to_item=None):
        """
        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (list) the path to the exception in the
                received_data dict. None if unset
        """

        self.path_to_item = path_to_item
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiValueError, self).__init__(full_msg)


class ApiAttributeError(OpenApiException, AttributeError):
    def __init__(self, msg, path_to_item=None):
        """
        Raised when an attribute reference or assignment fails.

        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (None/list) the path to the exception in the
                received_data dict
        """
        self.path_to_item = path_to_item
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiAttributeError, self).__init__(full_msg)


class ApiKeyError(OpenApiException, KeyError):
    def __init__(self, msg, path_to_item=None):
        """
        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (None/list) the path to the exception in the
                received_data dict
        """
        self.path_to_item = path_to_item
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiKeyError, self).__init__(full_msg)


class ApiException(OpenApiException):

    def __init__(self, status=None, reason=None, http_resp=None):
        if http_resp:
            self.status = http_resp.status
            self.reason = http_resp.reason
            self.body = http_resp.data
            self.headers = http_resp.getheaders()
        else:
            self.status = status
            self.reason = reason
            self.body = None
            self.headers = None

    def __str__(self):
        """Custom error messages for exception"""
        error_message = "Status Code: {0}\n"\
                        "Reason: {1}\n".format(self.status, self.reason)
        if self.headers:
            error_message += "HTTP response headers: {0}\n".format(
                self.headers)

        if self.body:
            error_message += "HTTP response body: {0}\n".format(self.body)

        return error_message


class NotFoundException(ApiException):

    def __init__(self, status=None, reason=None, http_resp=None):
        super(NotFoundException, self).__init__(status, reason, http_resp)


class UnauthorizedException(ApiException):

    def __init__(self, status=None, reason=None, http_resp=None):
        super(UnauthorizedException, self).__init__(status, reason, http_resp)


class ForbiddenException(ApiException):

    def __init__(self, status=None, reason=None, http_resp=None):
        super(ForbiddenException, self).__init__(status, reason, http_resp)


class ServiceException(ApiException):

    def __init__(self, status=None, reason=None, http_resp=None):
        super(ServiceException, self).__init__(status, reason, http_resp)


def render_path(path_to_item):
    """Returns a string representation of a path"""
    result = ""
    for pth in path_to_item:
        if isinstance(pth, int):
            result += "[{0}]".format(pth)
        else:
            result += "['{0}']".format(pth)
    return result



"""
    University Photo API

     ## Introduction  The Photo API allows access to Photos of members of the University. The Photos exposed by this API are expected to be a good representation of the person they relate to.  The API broadly follows the principles of REST and strives to provide an interface that can be easily consumed by downstream systems.  ### Stability  This release of the Photo API is a `beta` offering: a service we are moving towards live but which requires wider testing with a broader group of users.  ### Versioning  The Photo API is versioned using URL path prefixes in the format: `/v1beta1/photos`. This follows the pattern established by the [GCP API](https://cloud.google.com/apis/design/versioning). Breaking changes will not be made without a change in API major version, however non-breaking changes will be introduced without changes to the version path prefix. All changes will be documented in the project's [CHANGELOG](https://gitlab.developers.cam.ac.uk/uis/devops/iam/photo-api/-/blob/master/CHANGELOG.md)  The available versions of the API are listed at the API's root.  ### Domain  The Photo API serves Photos of members of the University, linking a Photo to an individual through `PhotoIdentifiers` (see below) which pertain to a person record within an external system.  The Photos served by the Photo API are expected to be of sufficient quality to identify the person they represent. Photos which have been reviewed and determined to be sufficient quality are marked as having the status `approved` and exposed through the `approved-photos` endpoints.  ### Data source  The Photos exposed by the Photo API are currently a mirror of card Photos contained within the [Card Database](https://webservices.admin.cam.ac.uk/uc/). With Photos being synced from the Card Database to the Photo API hourly.  In future, the Photo API will be extended to allow Photos to be created and 'approved' using the API.  ## Core entities  ### The `Photo` Entity  The `Photo` entity contains information and metadata about a Photo of a member of the university. The `status` field on a Photo indicates whether this Photo has been reviewed and approved. Only Photos which have the status `approved` are exposed through the `approved-photos` endpoints. Each photo entity has a unique identifier - termed UUID - which is used in the system to refer to the specific photo entity. The UUID is provided in JSON documents as the `id` property.   Each Photo entity contains a link to the image data itself and a thumbnail for this Photo, which is guaranteed to be less than 128x128 pixels. Images are served by redirecting to a temporary URL provided by the service used to host images. These temporary URLs will expire after an hour and should not be retained for long-lasting access to a given Photo.  Each Photo may contain a block of JSON metadata recording information about the Photo. Photos cannot be updated once created (apart from to move their status to `approved`) hence this metadata should not be used to store application state or any other information which is not related to the content of the Photo.  Each Photo will have a set of `PhotoIdentifier` which allow the Photo to be linked to a person record within another system.  ### The `PhotoIdentifier` Entity  The `PhotoIdentifier` entity holds the `value` and `scheme` of a given identifier. The `value` field of a `PhotoIdentifier` is a simple ID string - e.g. `wgd23` or `000001`. The `scheme` field of a `PhotoIdentifier` indicates what system this identifier relates to or was issued by. This allows many identifiers which relate to different systems to be recorded against a single `Photo`.  The supported schemes are: * `v1.person.identifiers.cam.ac.uk`: The CRSID of the person represented by this Photo * `person.v1.student-records.university.identifiers.cam.ac.uk`: The CamSIS identifier (USN) of the person represented by this Photo * `person.v1.human-resources.university.identifiers.cam.ac.uk`: The CHRIS identifier (staff number) of the person represented by this Photo * `photo.v1.legacy-card.university.identifiers.cam.ac.uk`: The id of this Photo within the legacy card system  ## Using the API  ### Auth  To authenticate against the Photo API, an application must be registered within the API Gateway and granted access to the `University Photo` product. Details of how to register an application and grant access to products can be found in the [API Gateway Getting Started Guide](https://developer.api.apps.cam.ac.uk/start-using-an-api).    # noqa: E501

    The version of the OpenAPI document: v1beta1
    Contact: devops+photoapi@uis.cam.ac.uk
    Generated by: https://openapi-generator.tech
"""


class OpenApiException(Exception):
    """The base exception class for all OpenAPIExceptions"""


class ApiTypeError(OpenApiException, TypeError):
    def __init__(self, msg, path_to_item=None, valid_classes=None,
                 key_type=None):
        """ Raises an exception for TypeErrors

        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (list): a list of keys an indices to get to the
                                 current_item
                                 None if unset
            valid_classes (tuple): the primitive classes that current item
                                   should be an instance of
                                   None if unset
            key_type (bool): False if our value is a value in a dict
                             True if it is a key in a dict
                             False if our item is an item in a list
                             None if unset
        """
        self.path_to_item = path_to_item
        self.valid_classes = valid_classes
        self.key_type = key_type
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiTypeError, self).__init__(full_msg)


class ApiValueError(OpenApiException, ValueError):
    def __init__(self, msg, path_to_item=None):
        """
        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (list) the path to the exception in the
                received_data dict. None if unset
        """

        self.path_to_item = path_to_item
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiValueError, self).__init__(full_msg)


class ApiAttributeError(OpenApiException, AttributeError):
    def __init__(self, msg, path_to_item=None):
        """
        Raised when an attribute reference or assignment fails.

        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (None/list) the path to the exception in the
                received_data dict
        """
        self.path_to_item = path_to_item
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiAttributeError, self).__init__(full_msg)


class ApiKeyError(OpenApiException, KeyError):
    def __init__(self, msg, path_to_item=None):
        """
        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (None/list) the path to the exception in the
                received_data dict
        """
        self.path_to_item = path_to_item
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiKeyError, self).__init__(full_msg)


class ApiException(OpenApiException):

    def __init__(self, status=None, reason=None, http_resp=None):
        if http_resp:
            self.status = http_resp.status
            self.reason = http_resp.reason
            self.body = http_resp.data
            self.headers = http_resp.getheaders()
        else:
            self.status = status
            self.reason = reason
            self.body = None
            self.headers = None

    def __str__(self):
        """Custom error messages for exception"""
        error_message = "Status Code: {0}\n"\
                        "Reason: {1}\n".format(self.status, self.reason)
        if self.headers:
            error_message += "HTTP response headers: {0}\n".format(
                self.headers)

        if self.body:
            error_message += "HTTP response body: {0}\n".format(self.body)

        return error_message


class NotFoundException(ApiException):

    def __init__(self, status=None, reason=None, http_resp=None):
        super(NotFoundException, self).__init__(status, reason, http_resp)


class UnauthorizedException(ApiException):

    def __init__(self, status=None, reason=None, http_resp=None):
        super(UnauthorizedException, self).__init__(status, reason, http_resp)


class ForbiddenException(ApiException):

    def __init__(self, status=None, reason=None, http_resp=None):
        super(ForbiddenException, self).__init__(status, reason, http_resp)


class ServiceException(ApiException):

    def __init__(self, status=None, reason=None, http_resp=None):
        super(ServiceException, self).__init__(status, reason, http_resp)


def render_path(path_to_item):
    """Returns a string representation of a path"""
    result = ""
    for pth in path_to_item:
        if isinstance(pth, int):
            result += "[{0}]".format(pth)
        else:
            result += "['{0}']".format(pth)
    return result



# flake8: noqa

# import all models into this package
# if you have many models here with many references from one model to another this may
# raise a RecursionError
# to avoid this, import only the models that you directly need like:
# from from identitylib.lookup_client.model.pet import Pet
# or import this package, but before doing it, use:
# import sys
# sys.setrecursionlimit(n)

from identitylib.lookup_client.model.attribute import Attribute
from identitylib.lookup_client.model.attribute_scheme import AttributeScheme
from identitylib.lookup_client.model.contact_phone_number import ContactPhoneNumber
from identitylib.lookup_client.model.contact_row import ContactRow
from identitylib.lookup_client.model.contact_web_page import ContactWebPage
from identitylib.lookup_client.model.error import Error
from identitylib.lookup_client.model.group import Group
from identitylib.lookup_client.model.group_all_groups200_response import GroupAllGroups200Response
from identitylib.lookup_client.model.group_all_groups200_response_result import GroupAllGroups200ResponseResult
from identitylib.lookup_client.model.group_all_groups_default_response import GroupAllGroupsDefaultResponse
from identitylib.lookup_client.model.group_all_groups_default_response_result import GroupAllGroupsDefaultResponseResult
from identitylib.lookup_client.model.group_get_cancelled_members200_response import GroupGetCancelledMembers200Response
from identitylib.lookup_client.model.group_get_cancelled_members200_response_result import GroupGetCancelledMembers200ResponseResult
from identitylib.lookup_client.model.group_get_group200_response import GroupGetGroup200Response
from identitylib.lookup_client.model.group_get_group200_response_result import GroupGetGroup200ResponseResult
from identitylib.lookup_client.model.group_members_of_inst import GroupMembersOfInst
from identitylib.lookup_client.model.group_search_count200_response import GroupSearchCount200Response
from identitylib.lookup_client.model.group_search_count200_response_result import GroupSearchCount200ResponseResult
from identitylib.lookup_client.model.ibis_get_last_transaction_id200_response import IbisGetLastTransactionId200Response
from identitylib.lookup_client.model.ibis_get_last_transaction_id200_response_result import IbisGetLastTransactionId200ResponseResult
from identitylib.lookup_client.model.ibis_get_version200_response import IbisGetVersion200Response
from identitylib.lookup_client.model.ibis_get_version200_response_result import IbisGetVersion200ResponseResult
from identitylib.lookup_client.model.identifier import Identifier
from identitylib.lookup_client.model.institution import Institution
from identitylib.lookup_client.model.institution_all_attribute_schemes200_response import InstitutionAllAttributeSchemes200Response
from identitylib.lookup_client.model.institution_all_attribute_schemes200_response_result import InstitutionAllAttributeSchemes200ResponseResult
from identitylib.lookup_client.model.institution_all_insts200_response import InstitutionAllInsts200Response
from identitylib.lookup_client.model.institution_all_insts200_response_result import InstitutionAllInsts200ResponseResult
from identitylib.lookup_client.model.institution_get_attribute200_response import InstitutionGetAttribute200Response
from identitylib.lookup_client.model.institution_get_attribute200_response_result import InstitutionGetAttribute200ResponseResult
from identitylib.lookup_client.model.institution_get_attributes200_response import InstitutionGetAttributes200Response
from identitylib.lookup_client.model.institution_get_attributes200_response_result import InstitutionGetAttributes200ResponseResult
from identitylib.lookup_client.model.institution_get_contact_rows200_response import InstitutionGetContactRows200Response
from identitylib.lookup_client.model.institution_get_contact_rows200_response_result import InstitutionGetContactRows200ResponseResult
from identitylib.lookup_client.model.institution_get_inst200_response import InstitutionGetInst200Response
from identitylib.lookup_client.model.institution_get_inst200_response_result import InstitutionGetInst200ResponseResult
from identitylib.lookup_client.model.person import Person
from identitylib.lookup_client.model.person_get_person200_response import PersonGetPerson200Response
from identitylib.lookup_client.model.person_get_person200_response_result import PersonGetPerson200ResponseResult
from identitylib.lookup_client.model.person_identifier import PersonIdentifier
from identitylib.lookup_client.model.person_is_member_of_group200_response import PersonIsMemberOfGroup200Response
from identitylib.lookup_client.model.person_is_member_of_group200_response_result import PersonIsMemberOfGroup200ResponseResult



# flake8: noqa

# import all models into this package
# if you have many models here with many references from one model to another this may
# raise a RecursionError
# to avoid this, import only the models that you directly need like:
# from from identitylib.card_client.model.pet import Pet
# or import this package, but before doing it, use:
# import sys
# sys.setrecursionlimit(n)

from identitylib.card_client.model.api_exception import APIException
from identitylib.card_client.model.available_barcode import AvailableBarcode
from identitylib.card_client.model.card import Card
from identitylib.card_client.model.card_identifier import CardIdentifier
from identitylib.card_client.model.card_logo import CardLogo
from identitylib.card_client.model.card_note import CardNote
from identitylib.card_client.model.card_request import CardRequest
from identitylib.card_client.model.card_request_distinct_values import CardRequestDistinctValues
from identitylib.card_client.model.card_request_summary import CardRequestSummary
from identitylib.card_client.model.card_summary import CardSummary
from identitylib.card_client.model.list200_response import List200Response
from identitylib.card_client.model.permissions_list200_response import PermissionsList200Response
from identitylib.card_client.model.v1beta1_available_barcodes_batch200_response import V1beta1AvailableBarcodesBatch200Response
from identitylib.card_client.model.v1beta1_available_barcodes_batch200_response_details import V1beta1AvailableBarcodesBatch200ResponseDetails
from identitylib.card_client.model.v1beta1_available_barcodes_batch200_response_details_invalid_inner import V1beta1AvailableBarcodesBatch200ResponseDetailsInvalidInner
from identitylib.card_client.model.v1beta1_available_barcodes_batch_request import V1beta1AvailableBarcodesBatchRequest
from identitylib.card_client.model.v1beta1_available_barcodes_create_request import V1beta1AvailableBarcodesCreateRequest
from identitylib.card_client.model.v1beta1_available_barcodes_list200_response import V1beta1AvailableBarcodesList200Response
from identitylib.card_client.model.v1beta1_card_logos_list200_response import V1beta1CardLogosList200Response
from identitylib.card_client.model.v1beta1_card_notes_create_request import V1beta1CardNotesCreateRequest
from identitylib.card_client.model.v1beta1_card_notes_delete200_response import V1beta1CardNotesDelete200Response
from identitylib.card_client.model.v1beta1_card_notes_delete_request import V1beta1CardNotesDeleteRequest
from identitylib.card_client.model.v1beta1_card_notes_list200_response import V1beta1CardNotesList200Response
from identitylib.card_client.model.v1beta1_card_requests_bulk_update200_response import V1beta1CardRequestsBulkUpdate200Response
from identitylib.card_client.model.v1beta1_card_requests_bulk_update200_response_details_inner import V1beta1CardRequestsBulkUpdate200ResponseDetailsInner
from identitylib.card_client.model.v1beta1_card_requests_bulk_update_request import V1beta1CardRequestsBulkUpdateRequest
from identitylib.card_client.model.v1beta1_card_requests_bulk_update_request_updates_inner import V1beta1CardRequestsBulkUpdateRequestUpdatesInner
from identitylib.card_client.model.v1beta1_card_requests_bulk_update_request_updates_inner_fields import V1beta1CardRequestsBulkUpdateRequestUpdatesInnerFields
from identitylib.card_client.model.v1beta1_card_requests_bulk_update_request_updates_inner_identifiers_inner import V1beta1CardRequestsBulkUpdateRequestUpdatesInnerIdentifiersInner
from identitylib.card_client.model.v1beta1_card_requests_create_request import V1beta1CardRequestsCreateRequest
from identitylib.card_client.model.v1beta1_card_requests_list200_response import V1beta1CardRequestsList200Response
from identitylib.card_client.model.v1beta1_card_requests_update_request import V1beta1CardRequestsUpdateRequest
from identitylib.card_client.model.v1beta1_card_rfid_data_config_list200_response import V1beta1CardRfidDataConfigList200Response
from identitylib.card_client.model.v1beta1_card_rfid_data_config_list200_response_results_inner import V1beta1CardRfidDataConfigList200ResponseResultsInner
from identitylib.card_client.model.v1beta1_cards_filter_request import V1beta1CardsFilterRequest
from identitylib.card_client.model.v1beta1_cards_list200_response import V1beta1CardsList200Response
from identitylib.card_client.model.v1beta1_cards_update_request import V1beta1CardsUpdateRequest
from identitylib.card_client.model.validation_error import ValidationError



from unittest import TestCase
from urllib3_mock import Responses
from base64 import b64encode
from datetime import datetime, timedelta
from freezegun import freeze_time

from identitylib.card_client_configuration import CardClientConfiguration
from identitylib.card_client import ApiClient
from identitylib.card_client.api.v1beta1_api import V1beta1Api as CardApi


responses = Responses('requests.packages.urllib3')


class TestApiClientConfigurationMixin(TestCase):

    @responses.activate
    def test_access_token_is_fetched(self):
        """ check the mixin requests an access token """
        def token_callback(request):
            self.assertEqual(
                request.headers.get('Authorization'),
                f'Basic {b64encode(b"client_key_test:client_secret_test").decode("utf-8")}'
            )
            return 201, {}, '{"access_token": "test_access_token"}'

        # mock the response from the token endpoint
        responses.add_callback('POST', '/oauth2/v1/token', token_callback)

        # mock a successful request for the available barcodes
        def available_barcodes_callback(request):
            self.assertEqual(request.headers.get('Authorization'), 'Bearer test_access_token')
            return 200, {}, '{"results": []}'
        responses.add_callback(
            'GET',
            '/card/v1beta1/available-barcodes',
            available_barcodes_callback
        )

        configuration = CardClientConfiguration('client_key_test', 'client_secret_test')
        card_api_instance = CardApi(ApiClient(configuration))

        card_api_instance.v1beta1_available_barcodes_list()

    @responses.activate
    def test_refresh_access_token_fails(self):
        """ check the mixin correctly raises exception when server responds 401 """

        # mock a 401 response code for the token
        responses.add('POST', '/oauth2/v1/token', status=401)

        configuration = CardClientConfiguration('client_key', 'client_secret')
        card_api_instance = CardApi(ApiClient(configuration))

        with self.assertRaises(RuntimeError):
            card_api_instance.v1beta1_available_barcodes_list()

    @responses.activate
    def test_refresh_access_token_no_token(self):
        """ check the mixin correctly raises exception when no access_token in response """

        responses.add('POST', '/oauth2/v1/token', body='{}')

        configuration = CardClientConfiguration('client_key', 'client_secret')
        card_api_instance = CardApi(ApiClient(configuration))

        with self.assertRaises(RuntimeError):
            card_api_instance.v1beta1_available_barcodes_list()

    @responses.activate
    def test_will_refresh_the_token(self):
        token_calls = 0

        # record requests for an access token
        def token_callback(request):
            nonlocal token_calls
            token_calls = token_calls + 1

            # respond with no 'expires_in' which means we use the default of 1 minute
            return 200, {}, '{"access_token": "new_access_token"}'

        responses.add_callback('POST', '/oauth2/v1/token', token_callback)

        # mock a successful request for the available barcodes
        responses.add('GET', '/card/v1beta1/available-barcodes', body='{"results": []}')

        configuration = CardClientConfiguration('client_key_test', 'client_secret_test')
        card_api_instance = CardApi(ApiClient(configuration))

        now = datetime.now()
        with freeze_time(now) as frozen_datetime:
            card_api_instance.v1beta1_available_barcodes_list()
            # an access token should have been requested
            self.assertEqual(token_calls, 1)

            frozen_datetime.move_to(now + timedelta(seconds=60))
            card_api_instance.v1beta1_available_barcodes_list()

            # another access token should have been requested
            self.assertEqual(token_calls, 2)

            card_api_instance.v1beta1_available_barcodes_list()
            # our access token is still valid so we shouldn't have requested another
            self.assertEqual(token_calls, 2)

    @responses.activate
    def test_will_use_cached_token_until_expired(self):
        token_calls = 0

        # record requests for an access token
        def token_callback(request):
            nonlocal token_calls
            token_calls = token_calls + 1

            # respond with an 'expires_in' which indicates the token expires in 10 minutes
            return 200, {}, '{"access_token": "test_access_token", "expires_in": 600}'

        responses.add_callback('POST', '/oauth2/v1/token', token_callback)

        # mock a successful request for the available barcodes
        responses.add('GET', '/card/v1beta1/available-barcodes', body='{"results": []}')

        configuration = CardClientConfiguration('client_key_test', 'client_secret_test')

        card_api_instance = CardApi(ApiClient(configuration))

        now = datetime.now()
        with freeze_time(now) as frozen_datetime:
            # first two calls should use the existing API token
            card_api_instance.v1beta1_available_barcodes_list()
            card_api_instance.v1beta1_available_barcodes_list()

            self.assertEqual(token_calls, 1)

            # travel forward in time
            frozen_datetime.move_to(now + timedelta(seconds=600))

            card_api_instance.v1beta1_available_barcodes_list()
            # a new access token should have been requested, so we now have two calls
            self.assertEqual(token_calls, 2)



from typing import Optional, List, Callable, Dict
from collections import namedtuple
from dataclasses import dataclass
from functools import lru_cache
from frozendict import frozendict


@dataclass(frozen=True, eq=True)
class IdentifierScheme:
    """
    A class which containing the information needed to represent an identifier scheme.

    """

    identifier: str  # the actual string which identifiers this scheme
    common_name: str  # a common name for this identifier (e.g. CRSid or USN)
    # a dict of aliases - each value being a previous representation of this identifier
    # keyed on a brief description of that alias
    aliases: Dict[str, str]
    value_parser: Optional[Callable] = None  # a parser for the value of an id with this scheme

    def __str__(self):
        """
        Always return the scheme identifier as the string representation.

        """
        return self.identifier


class IdentifierSchemes:
    """
    A holder for all identifier schemes used within identity systems.

    """

    """
    Person identifiers.

    """

    CRSID = IdentifierScheme(
        'v1.person.identifiers.cam.ac.uk',
        'CRSid',
        frozendict({
            'deprecated': 'person.crs.identifiers.uis.cam.ac.uk',
            'deprecated-versioned': 'person.v1.crs.identifiers.cam.ac.uk',
        })
    )

    USN = IdentifierScheme(
        'person.v1.student-records.university.identifiers.cam.ac.uk',
        'USN',
        frozendict({
            'deprecated': 'person.camsis.identifiers.admin.cam.ac.uk',
            'deprecated-versioned': 'person.v1.ust.identifiers.cam.ac.uk',
        })
    )

    STAFF_NUMBER = IdentifierScheme(
        'person.v1.human-resources.university.identifiers.cam.ac.uk',
        'Staff Number',
        frozendict({
            'deprecated': 'person.chris.identifiers.admin.cam.ac.uk',
            'deprecated-versioned': 'person.v1.uhr.identifiers.cam.ac.uk',
        })
    )

    BOARD_OF_GRADUATE_STUDIES = IdentifierScheme(
        'person.v1.board-of-graduate-studies.university.identifiers.cam.ac.uk',
        'Board of Graduate Studies Identifier',
        frozendict({
            'deprecated': 'person.bgs.identifiers.admin.cam.ac.uk',
        })
    )

    LEGACY_CARDHOLDER = IdentifierScheme(
        'person.v1.legacy-card.university.identifiers.cam.ac.uk',
        'Legacy cardholder Identifier',
        frozendict({
            'deprecated': 'person.legacy_card.identifiers.admin.cam.ac.uk',
        })
    )

    """
    Institution identifiers.

    """

    STUDENT_INSTITUTION = IdentifierScheme(
        'institution.v1.student-records.university.identifiers.cam.ac.uk',
        'Student Institution',
        frozendict({
            'deprecated': 'institution.v1.ust.identifiers.cam.ac.uk',
            'deprecated-mapping': 'institution.v1.student.university.identifiers.cam.ac.uk',
        })
    )

    HR_INSTITUTION = IdentifierScheme(
        'institution.v1.human-resources.university.identifiers.cam.ac.uk',
        'Human Resources Institution',
        frozendict({
            'deprecated': 'institution.v1.uhr.identifiers.cam.ac.uk',
        })
    )

    LEGACY_CARD_INSTITUTION = IdentifierScheme(
        'institution.v1.legacy-card.university.identifiers.cam.ac.uk',
        'Legacy Card Institution',
        frozendict({
            'deprecated': 'inst.legacy_card.identifiers.admin.cam.ac.uk',
        })
    )

    LOOKUP_INSTITUTION = IdentifierScheme(
        'insts.lookup.cam.ac.uk',
        'Lookup Institution',
        frozendict({})
    )

    """
    Misc. identifiers.

    """

    STUDENT_ACADEMIC_PLAN = IdentifierScheme(
        'academic-plan.v1.student-records.university.identifiers.cam.ac.uk',
        'Student Academic Plan',
        frozendict({
            'deprecated': 'academicPlan.v1.ust.identifiers.cam.ac.uk',
        })
    )

    CARD = IdentifierScheme(
        'card.v1.card.university.identifiers.cam.ac.uk',
        'Card Identifier',
        frozendict({
            'deprecated': 'card.card.identifiers.uis.cam.ac.uk',
        })
    )

    LEGACY_TEMP_CARD = IdentifierScheme(
        'temporary-card.v1.card.university.identifiers.cam.ac.uk',
        'Temporary Card Identifier',
        frozendict({
            'deprecated': 'temp_id.card.identifiers.uis.cam.ac.uk',
        })
    )

    MIFARE_ID = IdentifierScheme(
        'mifare-identifier.v1.card.university.identifiers.cam.ac.uk',
        'Mifare Identifier',
        frozendict({
            'deprecated': 'mifare_id.card.identifiers.uis.cam.ac.uk',
        }),
        value_parser=(
            lambda v: v.lstrip('0') or '0'  # fallback to '0' to avoid stripping '000' to ''
        )
    )

    MIFARE_NUMBER = IdentifierScheme(
        'mifare-number.v1.card.university.identifiers.cam.ac.uk',
        'Mifare Number',
        frozendict({
            'deprecated': 'mifare_number.card.identifiers.uis.cam.ac.uk',
        })
    )

    BARCODE = IdentifierScheme(
        'barcode.v1.card.university.identifiers.cam.ac.uk',
        'Card Barcode',
        frozendict({
            'deprecated': 'barcode.identifiers.lib.cam.ac.uk',
        })
    )

    CARD_LOGO = IdentifierScheme(
        'card-logo.v1.card.university.identifiers.cam.ac.uk',
        'Card Logo Identifier',
        frozendict({
            'deprecated': 'card_logo.card.identifiers.uis.cam.ac.uk',
        })
    )

    PHOTO = IdentifierScheme(
        'photo.v1.photo.university.identifiers.cam.ac.uk',
        'Photo Identifier',
        frozendict({
            'deprecated': 'photo_id.photo.identifiers.uis.cam.ac.uk',
        })
    )

    LEGACY_PHOTO = IdentifierScheme(
        'photo.v1.legacy-card.university.identifiers.cam.ac.uk',
        'Legacy Photo Identifier',
        frozendict({
            'deprecated': 'photo.legacy_card.identifiers.admin.cam.ac.uk',
        })
    )

    LEGACY_CARD = IdentifierScheme(
        'card.v1.legacy-card.university.identifiers.cam.ac.uk',
        'Legacy Card Identifier',
        frozendict({
            'deprecated': 'card.legacy_card.identifiers.admin.cam.ac.uk',
        })
    )

    LOOKUP_GROUP = IdentifierScheme(
        'groups.lookup.cam.ac.uk',
        'Lookup Group Identifier',
        frozendict({})
    )

    API_GATEWAY_APPLICATION = IdentifierScheme(
        'application.api.apps.cam.ac.uk',
        'Application Gateway Application',
        frozendict({
            'development': 'apigee-development.devel.api.gcp.uis.cam.ac.uk',
            'staging': 'apigee-staging.test.api.gcp.uis.cam.ac.uk'
        })
    )

    @staticmethod
    @lru_cache()
    def get_registered_schemes() -> List[IdentifierScheme]:
        """
        Returns the list of registered identifier schemes.

        """
        return [
            prop for prop in vars(IdentifierSchemes).values() if (
                isinstance(prop, IdentifierScheme)
            )
        ]

    @staticmethod
    def from_string(identifier_scheme: str, find_by_alias: bool = False):
        """
        Return an instance of an identifier scheme from a string representation.
        If `find_by_alias` is true identifier schemes will be matched based on
        the alias as well as.

        """
        matching_scheme = next((
            scheme for scheme in IdentifierSchemes.get_registered_schemes() if (
                scheme.identifier == identifier_scheme or
                find_by_alias and identifier_scheme in scheme.aliases.values()
            )
        ), None)

        if not matching_scheme:
            raise ValueError(f'Invalid identifier scheme {identifier_scheme}')

        return matching_scheme


class Identifier(namedtuple('Identifier', ['value', 'scheme'])):
    """
    A representation of an identifer, in the form of 'value' and 'scheme'

    """

    @staticmethod
    def from_string(
        value: str, *,
        fallback_scheme: Optional[IdentifierScheme] = None,
        find_by_alias: bool = False,
    ) -> 'Identifier':
        """
        Parse a `<value>@<scheme>` string into an identifier pair.

        """
        parsed_value: Optional[str] = None
        scheme: Optional[IdentifierScheme] = None

        parts = value.split('@')
        if len(parts) == 2:
            parsed_value = parts[0]
            scheme = IdentifierSchemes.from_string(parts[1], find_by_alias)
        elif len(parts) == 1 and fallback_scheme is not None:
            parsed_value = value
            scheme = fallback_scheme
        else:
            raise ValueError(f'Invalid identifier {value}')

        parsed_value = scheme.value_parser(parsed_value) if scheme.value_parser else parsed_value
        return Identifier(parsed_value, scheme)

    def __str__(self):
        """
        Parse an identifier back to string form.

        """
    # Always deal with identifiers in lower case
    # The case of the identifier used does not matter when calling Lookup or the Card API
    # but when using identifiers as keys within dicts we should ensure that we don't
    # accidentally create duplicates by having identifiers in mixed cases

        return f'{self.value}@{self.scheme}'.lower()


CRSID_SCHEME = str(IdentifierSchemes.CRSID)



"""
    Lookup/Ibis web service API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.2
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from identitylib.lookup_client.model_utils import (  # noqa: F401
    ApiTypeError,
    ModelComposed,
    ModelNormal,
    ModelSimple,
    cached_property,
    change_keys_js_to_python,
    convert_js_args_to_python_args,
    date,
    datetime,
    file_type,
    none_type,
    validate_get_composed_info,
    OpenApiModel
)
from identitylib.lookup_client.exceptions import ApiAttributeError


def lazy_import():
    from identitylib.lookup_client.model.group_members_of_inst import GroupMembersOfInst
    from identitylib.lookup_client.model.institution import Institution
    from identitylib.lookup_client.model.person import Person
    globals()['GroupMembersOfInst'] = GroupMembersOfInst
    globals()['Institution'] = Institution
    globals()['Person'] = Person


class Group(ModelNormal):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Attributes:
      allowed_values (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          with a capitalized key describing the allowed value and an allowed
          value. These dicts store the allowed enum values.
      attribute_map (dict): The key is attribute name
          and the value is json key in definition.
      discriminator_value_class_map (dict): A dict to go from the discriminator
          variable value to the discriminator class name.
      validations (dict): The key is the tuple path to the attribute
          and the for var_name this is (var_name,). The value is a dict
          that stores validations for max_length, min_length, max_items,
          min_items, exclusive_maximum, inclusive_maximum, exclusive_minimum,
          inclusive_minimum, and regex.
      additional_properties_type (tuple): A tuple of classes accepted
          as additional properties values.
    """

    allowed_values = {
    }

    validations = {
    }

    @cached_property
    def additional_properties_type():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded
        """
        lazy_import()
        return (bool, date, datetime, dict, float, int, list, str, none_type,)  # noqa: E501

    _nullable = False

    @cached_property
    def openapi_types():
        """
        This must be a method because a model may have properties that are
        of type self, this must run after the class is loaded

        Returns
            openapi_types (dict): The key is attribute name
                and the value is attribute type.
        """
        lazy_import()
        return {
            'cancelled': (bool,),  # noqa: E501
            'groupid': (str,),  # noqa: E501
            'name': (str,),  # noqa: E501
            'title': (str,),  # noqa: E501
            'description': (str,),  # noqa: E501
            'email': (str,),  # noqa: E501
            'members_of_inst': (GroupMembersOfInst,),  # noqa: E501
            'members': ([Person],),  # noqa: E501
            'direct_members': ([Person],),  # noqa: E501
            'owning_insts': ([Institution],),  # noqa: E501
            'manages_insts': ([Institution],),  # noqa: E501
            'manages_groups': ([Group],),  # noqa: E501
            'managed_by_groups': ([Group],),  # noqa: E501
            'reads_groups': ([Group],),  # noqa: E501
            'read_by_groups': ([Group],),  # noqa: E501
            'includes_groups': ([Group],),  # noqa: E501
            'included_by_groups': ([Group],),  # noqa: E501
            'id': (str,),  # noqa: E501
            'ref': (str,),  # noqa: E501
        }

    @cached_property
    def discriminator():
        return None


    attribute_map = {
        'cancelled': 'cancelled',  # noqa: E501
        'groupid': 'groupid',  # noqa: E501
        'name': 'name',  # noqa: E501
        'title': 'title',  # noqa: E501
        'description': 'description',  # noqa: E501
        'email': 'email',  # noqa: E501
        'members_of_inst': 'membersOfInst',  # noqa: E501
        'members': 'members',  # noqa: E501
        'direct_members': 'directMembers',  # noqa: E501
        'owning_insts': 'owningInsts',  # noqa: E501
        'manages_insts': 'managesInsts',  # noqa: E501
        'manages_groups': 'managesGroups',  # noqa: E501
        'managed_by_groups': 'managedByGroups',  # noqa: E501
        'reads_groups': 'readsGroups',  # noqa: E501
        'read_by_groups': 'readByGroups',  # noqa: E501
        'includes_groups': 'includesGroups',  # noqa: E501
        'included_by_groups': 'includedByGroups',  # noqa: E501
        'id': 'id',  # noqa: E501
        'ref': 'ref',  # noqa: E501
    }

    read_only_vars = {
    }

    _composed_schemas = {}

    @classmethod
    @convert_js_args_to_python_args
    def _from_openapi_data(cls, *args, **kwargs):  # noqa: E501
        """Group - a model defined in OpenAPI

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            cancelled (bool): Flag indicating if the group is cancelled.. [optional]  # noqa: E501
            groupid (str): The group's numeric ID (actually a string e.g., \"100656\").. [optional]  # noqa: E501
            name (str): The group's unique name (e.g., \"cs-editors\").. [optional]  # noqa: E501
            title (str): The group's title.. [optional]  # noqa: E501
            description (str): The more detailed description of the group.. [optional]  # noqa: E501
            email (str): The group's email address.. [optional]  # noqa: E501
            members_of_inst (GroupMembersOfInst): [optional]  # noqa: E501
            members ([Person]): A list of the group's members, including (recursively) any members of any included groups. This will only be populated if the `fetch` parameter includes the `\"all_members\"` option.. [optional]  # noqa: E501
            direct_members ([Person]): A list of the group's direct members, not including any members included via groups included by this group. This will only be populated if the `fetch` parameter includes the `\"direct_members\"` option.. [optional]  # noqa: E501
            owning_insts ([Institution]): A list of the institutions to which this group belongs. This will only be populated if the `fetch` parameter includes the `\"owning_insts\"` option.. [optional]  # noqa: E501
            manages_insts ([Institution]): A list of the institutions managed by this group. This will only be populated if the `fetch` parameter includes the `\"manages_insts\"` option.. [optional]  # noqa: E501
            manages_groups ([Group]): A list of the groups managed by this group. This will only be populated if the `fetch` parameter includes the `\"manages_groups\"` option.. [optional]  # noqa: E501
            managed_by_groups ([Group]): A list of the groups that manage this group. This will only be populated if the `fetch` parameter includes the `\"managed_by_groups\"` option.. [optional]  # noqa: E501
            reads_groups ([Group]): A list of the groups that this group has privileged access to. Members of this group will be able to read the members of any of those groups, regardless of the membership visibilities. This will only be populated if the `fetch` parameter includes the `\"reads_groups\"` option.. [optional]  # noqa: E501
            read_by_groups ([Group]): A list of the groups that have privileged access to this group. Members of those groups will be able to read the members of this group, regardless of the membership visibilities. This will only be populated if the `fetch` parameter includes the `\"read_by_groups\"` option.. [optional]  # noqa: E501
            includes_groups ([Group]): A list of the groups directly included in this group. Any members of the included groups (and recursively any groups that they include) will automatically be included in this group. This will only be populated if the `fetch` parameter includes the `\"includes_groups\"` option.. [optional]  # noqa: E501
            included_by_groups ([Group]): A list of the groups that directly include this group. Any members of this group will automatically be included in those groups (and recursively in any groups that include those groups). This will only be populated if the `fetch` parameter includes the `\"included_by_groups\"` option.. [optional]  # noqa: E501
            id (str): An ID that can uniquely identify this group within the returned XML/JSON document. This is only used in the flattened XML/JSON representation (if the \"flatten\" parameter is specified).. [optional]  # noqa: E501
            ref (str): A reference (by id) to a group element in the XML/JSON document. This is only used in the flattened XML/JSON representation (if the \"flatten\" parameter is specified).. [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', True)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        self = super(OpenApiModel, cls).__new__(cls)

        if args:
            for arg in args:
                if isinstance(arg, dict):
                    kwargs.update(arg)
                else:
                    raise ApiTypeError(
                        "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                            args,
                            self.__class__.__name__,
                        ),
                        path_to_item=_path_to_item,
                        valid_classes=(self.__class__,),
                    )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
        return self

    required_properties = set([
        '_data_store',
        '_check_type',
        '_spec_property_naming',
        '_path_to_item',
        '_configuration',
        '_visited_composed_classes',
    ])

    @convert_js_args_to_python_args
    def __init__(self, *args, **kwargs):  # noqa: E501
        """Group - a model defined in OpenAPI

        Keyword Args:
            _check_type (bool): if True, values for parameters in openapi_types
                                will be type checked and a TypeError will be
                                raised if the wrong type is input.
                                Defaults to True
            _path_to_item (tuple/list): This is a list of keys or values to
                                drill down to the model in received_data
                                when deserializing a response
            _spec_property_naming (bool): True if the variable names in the input data
                                are serialized names, as specified in the OpenAPI document.
                                False if the variable names in the input data
                                are pythonic names, e.g. snake case (default)
            _configuration (Configuration): the instance to use when
                                deserializing a file_type parameter.
                                If passed, type conversion is attempted
                                If omitted no type conversion is done.
            _visited_composed_classes (tuple): This stores a tuple of
                                classes that we have traveled through so that
                                if we see that class again we will not use its
                                discriminator again.
                                When traveling through a discriminator, the
                                composed schema that is
                                is traveled through is added to this set.
                                For example if Animal has a discriminator
                                petType and we pass in "Dog", and the class Dog
                                allOf includes Animal, we move through Animal
                                once using the discriminator, and pick Dog.
                                Then in Dog, we will make an instance of the
                                Animal class but this time we won't travel
                                through its discriminator because we passed in
                                _visited_composed_classes = (Animal,)
            cancelled (bool): Flag indicating if the group is cancelled.. [optional]  # noqa: E501
            groupid (str): The group's numeric ID (actually a string e.g., \"100656\").. [optional]  # noqa: E501
            name (str): The group's unique name (e.g., \"cs-editors\").. [optional]  # noqa: E501
            title (str): The group's title.. [optional]  # noqa: E501
            description (str): The more detailed description of the group.. [optional]  # noqa: E501
            email (str): The group's email address.. [optional]  # noqa: E501
            members_of_inst (GroupMembersOfInst): [optional]  # noqa: E501
            members ([Person]): A list of the group's members, including (recursively) any members of any included groups. This will only be populated if the `fetch` parameter includes the `\"all_members\"` option.. [optional]  # noqa: E501
            direct_members ([Person]): A list of the group's direct members, not including any members included via groups included by this group. This will only be populated if the `fetch` parameter includes the `\"direct_members\"` option.. [optional]  # noqa: E501
            owning_insts ([Institution]): A list of the institutions to which this group belongs. This will only be populated if the `fetch` parameter includes the `\"owning_insts\"` option.. [optional]  # noqa: E501
            manages_insts ([Institution]): A list of the institutions managed by this group. This will only be populated if the `fetch` parameter includes the `\"manages_insts\"` option.. [optional]  # noqa: E501
            manages_groups ([Group]): A list of the groups managed by this group. This will only be populated if the `fetch` parameter includes the `\"manages_groups\"` option.. [optional]  # noqa: E501
            managed_by_groups ([Group]): A list of the groups that manage this group. This will only be populated if the `fetch` parameter includes the `\"managed_by_groups\"` option.. [optional]  # noqa: E501
            reads_groups ([Group]): A list of the groups that this group has privileged access to. Members of this group will be able to read the members of any of those groups, regardless of the membership visibilities. This will only be populated if the `fetch` parameter includes the `\"reads_groups\"` option.. [optional]  # noqa: E501
            read_by_groups ([Group]): A list of the groups that have privileged access to this group. Members of those groups will be able to read the members of this group, regardless of the membership visibilities. This will only be populated if the `fetch` parameter includes the `\"read_by_groups\"` option.. [optional]  # noqa: E501
            includes_groups ([Group]): A list of the groups directly included in this group. Any members of the included groups (and recursively any groups that they include) will automatically be included in this group. This will only be populated if the `fetch` parameter includes the `\"includes_groups\"` option.. [optional]  # noqa: E501
            included_by_groups ([Group]): A list of the groups that directly include this group. Any members of this group will automatically be included in those groups (and recursively in any groups that include those groups). This will only be populated if the `fetch` parameter includes the `\"included_by_groups\"` option.. [optional]  # noqa: E501
            id (str): An ID that can uniquely identify this group within the returned XML/JSON document. This is only used in the flattened XML/JSON representation (if the \"flatten\" parameter is specified).. [optional]  # noqa: E501
            ref (str): A reference (by id) to a group element in the XML/JSON document. This is only used in the flattened XML/JSON representation (if the \"flatten\" parameter is specified).. [optional]  # noqa: E501
        """

        _check_type = kwargs.pop('_check_type', True)
        _spec_property_naming = kwargs.pop('_spec_property_naming', False)
        _path_to_item = kwargs.pop('_path_to_item', ())
        _configuration = kwargs.pop('_configuration', None)
        _visited_composed_classes = kwargs.pop('_visited_composed_classes', ())

        if args:
            for arg in args:
                if isinstance(arg, dict):
                    kwargs.update(arg)
                else:
                    raise ApiTypeError(
                        "Invalid positional arguments=%s passed to %s. Remove those invalid positional arguments." % (
                            args,
                            self.__class__.__name__,
                        ),
                        path_to_item=_path_to_item,
                        valid_classes=(self.__class__,),
                    )

        self._data_store = {}
        self._check_type = _check_type
        self._spec_property_naming = _spec_property_naming
        self._path_to_item = _path_to_item
        self._configuration = _configuration
        self._visited_composed_classes = _visited_composed_classes + (self.__class__,)

        for var_name, var_value in kwargs.items():
            if var_name not in self.attribute_map and \
                        self._configuration is not None and \
                        self._configuration.discard_unknown_keys and \
                        self.additional_properties_type is None:
                # discard variable.
                continue
            setattr(self, var_name, var_value)
            if var_name in self.read_only_vars:
                raise ApiAttributeError(f"`{var_name}` is a read-only attribute. Use `from_openapi_data` to instantiate "
                                     f"class with read only attributes.")



"""
    University Student API

    This API exposes identity information about university students. The purpose of this API is to give access to basic identity information about university students for use by downstream systems, therefore the data exposed is limited to basic information about who a student is and which institutions they are affiliated with.  This API uses the following identifiers to link people to person-records held within different systems:  * `person.v1.student-records.university.identifiers.cam.ac.uk` - the university student (ust) identifier of this person -   also referred to as USN  Additionally this API uses the following identifiers to link people to institutions or entities which a person may have an affiliation to:  * `institution.v1.student-records.university.identifiers.cam.ac.uk` - the university student (ust) institution identifier.     This encompasses colleges, departments and faculties. The     [CamSIS Coding Manual](https://www.camsis.cam.ac.uk/technical/camsis-coding-manual)     lists the available values for this affiliation (under A01 and M01). * `academic-plan.v1.student-records.university.identifiers.cam.ac.uk` - the university student (ust) academic plan     identifier. The     [CamSIS Coding Manual](https://www.camsis.cam.ac.uk/technical/camsis-coding-manual)     lists the available values for this affiliation (under D07).  Both affiliations provide a `status` field which indicate a student's status within an affiliated entity. This `status` field is the short representation of the academic career field held in CamSIS. The [CamSIS Coding Manual](https://www.camsis.cam.ac.uk/technical/camsis-coding-manual) lists the available values for the `status` field (under D05).  This API is currently backed by data from CamSIS, exposing a view of data that is refreshed nightly.    # noqa: E501

    The version of the OpenAPI document: v1alpha2
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from identitylib.student_client.api_client import ApiClient, Endpoint as _Endpoint
from identitylib.student_client.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from identitylib.student_client.model.http_exception import HTTPException
from identitylib.student_client.model.http_validation_error import HTTPValidationError
from identitylib.student_client.model.paginated_results_student import PaginatedResultsStudent
from identitylib.student_client.model.student import Student


class StudentsApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client
        self.get_v1alpha2_student_students_identifier_get_endpoint = _Endpoint(
            settings={
                'response_type': (Student,),
                'auth': [
                    'ApiGatewayOAuthClientCredentials'
                ],
                'endpoint_path': '/students/{identifier}',
                'operation_id': 'get_v1alpha2_student_students_identifier_get',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'identifier',
                ],
                'required': [
                    'identifier',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'identifier':
                        (str,),
                },
                'attribute_map': {
                    'identifier': 'identifier',
                },
                'location_map': {
                    'identifier': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.list_v1alpha2_recent_graduates_recent_graduates_get_endpoint = _Endpoint(
            settings={
                'response_type': (PaginatedResultsStudent,),
                'auth': [
                    'ApiGatewayOAuthClientCredentials'
                ],
                'endpoint_path': '/recent-graduates',
                'operation_id': 'list_v1alpha2_recent_graduates_recent_graduates_get',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'affiliation',
                    'cursor',
                ],
                'required': [],
                'nullable': [
                    'affiliation',
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'affiliation':
                        (str, none_type,),
                    'cursor':
                        (str,),
                },
                'attribute_map': {
                    'affiliation': 'affiliation',
                    'cursor': 'cursor',
                },
                'location_map': {
                    'affiliation': 'query',
                    'cursor': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.list_v1alpha2_students_students_get_endpoint = _Endpoint(
            settings={
                'response_type': (PaginatedResultsStudent,),
                'auth': [
                    'ApiGatewayOAuthClientCredentials'
                ],
                'endpoint_path': '/students',
                'operation_id': 'list_v1alpha2_students_students_get',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'affiliation',
                    'cursor',
                ],
                'required': [],
                'nullable': [
                    'affiliation',
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'affiliation':
                        (str, none_type,),
                    'cursor':
                        (str,),
                },
                'attribute_map': {
                    'affiliation': 'affiliation',
                    'cursor': 'cursor',
                },
                'location_map': {
                    'affiliation': 'query',
                    'cursor': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )

    def get_v1alpha2_student_students_identifier_get(
        self,
        identifier,
        **kwargs
    ):
        """Get a single student  # noqa: E501

        Returns a single student based on the identifier provided. The identifier can be provided in the format `<value>@<scheme>`, if the scheme is omitted it shall be assumed that the scheme is `person.v1.student-records.university.identifiers.cam.ac.uk` (also know as the USN).   Currently this endpoint only supports querying by `person.v1.student-records.university.identifiers.cam.ac.uk` identifiers, but may be extended to allow querying by additional identifier schemes in future.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_v1alpha2_student_students_identifier_get(identifier, async_req=True)
        >>> result = thread.get()

        Args:
            identifier (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            Student
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['identifier'] = \
            identifier
        return self.get_v1alpha2_student_students_identifier_get_endpoint.call_with_http_info(**kwargs)

    def list_v1alpha2_recent_graduates_recent_graduates_get(
        self,
        **kwargs
    ):
        """List students who have completed a course within the last six months  # noqa: E501

        Returns students who have ended their affiliation with an institution or academic plan within the last six months, indicating that they have recently completed their studies. This is to allow downstream systems access to data on students who may in-practice continue to have an affiliation with the University, even though their course has technically ended.  Affiliations populated on each recent graduate are limited to the affiliations which have ended within the last six months. Graduates may have additional affiliations which are not represented here. These affiliations can be accessed by querying the `/students` endpoint.  Students who have either been removed or withdrawn from a course are not returned.  To avoid returning very large responses, this endpoint produces a paged response. Where there are additional results which cannot be returned on a single page, a `next` field will be populated with a url containing the next page of results. These `next` urls should be followed until a page is returned with no `next` field, indicating that there are no further results. Each page returned will hold a maximum of 1000 results.  Graduates can be filtered by affiliation, by providing an `affiliation` query parameter. This affiliation should be provided in the format `<value>@<scheme>`. Graduates will only be returned if their affiliation with an institution or academic plan has ended within the last six months.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_v1alpha2_recent_graduates_recent_graduates_get(async_req=True)
        >>> result = thread.get()


        Keyword Args:
            affiliation (str, none_type): An affiliation to filter by, in the format `<value>@<scheme>`. When included only graduates with a matching affiliation are returned.. [optional]
            cursor (str): The cursor indicating a unique page of results - this should be auto generated on the `next` and `previous` fields and does not need to be manually added / updated.. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            PaginatedResultsStudent
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        return self.list_v1alpha2_recent_graduates_recent_graduates_get_endpoint.call_with_http_info(**kwargs)

    def list_v1alpha2_students_students_get(
        self,
        **kwargs
    ):
        """List students  # noqa: E501

        Lists all students held within the University Student management system, allowing basic filtering by affiliation.  To avoid returning very large responses, this endpoint produces a paged response. Where there are additional results which cannot be returned on a single page, a `next` field will be populated with a url containing the next page of results. These `next` urls should be followed until a page is returned with no `next` field, indicating that there are no further results. Each page returned will hold a maximum of 1000 results.  Students can be filtered by affiliation, by providing an `affiliation` query parameter. This affiliation should be provided in the format `<value>@<scheme>`.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.list_v1alpha2_students_students_get(async_req=True)
        >>> result = thread.get()


        Keyword Args:
            affiliation (str, none_type): An affiliation to filter by, in the format `<value>@<scheme>`. When included only the students with a matching affiliation are returned.. [optional]
            cursor (str): The cursor indicating a unique page of results - this should be auto generated on the `next` and `previous` fields and does not need to be manually added / updated.. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            PaginatedResultsStudent
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        return self.list_v1alpha2_students_students_get_endpoint.call_with_http_info(**kwargs)




import requests
from typing import Optional
from logging import getLogger
from datetime import datetime, timedelta

LOG = getLogger(__name__)


class ClientCredentialsConfigurationMixin:
    """
    Mixin to support client_credentials authentication flow.

    """

    def __init__(self, client_key, client_secret, access_token_url=None):
        self.client_key = client_key
        self.client_secret = client_secret
        self.access_token_url = access_token_url

        # a holder for our cached access token
        self._access_token: Optional[str] = None
        # a holder for the timestamp at which our access token expires
        self._access_token_expires_at: Optional[datetime] = None

    def auth_settings(self):
        """
        Overide the auth settings method to ensure that the access token is updated
        before being used in an API request.


        """
        self.access_token = self._get_api_gateway_access_token()
        return super().auth_settings()

    def _get_api_gateway_access_token(self):
        """
        Returns either a recently refreshed access token or the cached access token
        if still valid.

        """
        now = datetime.now()
        if (
            self._access_token is not None and
            self._access_token_expires_at is not None and
            # ensure that our access token is still valid - with a leeway of 20 seconds
            now < (self._access_token_expires_at - timedelta(seconds=20))
        ):
            return self._access_token

        LOG.info("Refreshing API Gateway access token")

        auth = requests.auth.HTTPBasicAuth(self.client_key, self.client_secret)
        data = {"grant_type": "client_credentials"}

        response = requests.post(self.access_token_url, data=data, auth=auth)
        if (
            response.status_code >= 300 or
            response.status_code < 200 or
            not response.json().get('access_token')
        ):
            raise RuntimeError(
                f'Access token request failed: {response.status_code}, {response.text}'
            )

        response_data = response.json()
        self._access_token = response_data['access_token']
        # the `expires_in` value is in seconds rather than milliseconds, if we don't have
        # an `expires_in` use 1 minute as a reasonable default.
        self._access_token_expires_at = now + timedelta(
            seconds=response_data.get('expires_in', 60)
        )

        return self._access_token
